"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .canonicalizingregextype import (
    CanonicalizingRegexType,
    CanonicalizingRegexTypeTypedDict,
)
from .objectdefinition import ObjectDefinition, ObjectDefinitionTypedDict
from .quicklink import Quicklink, QuicklinkTypedDict
from enum import Enum
from glean.types import BaseModel
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class DatasourceCategory(str, Enum):
    r"""The type of this datasource. It is an important signal for relevance and must be specified and cannot be UNCATEGORIZED. Please refer to [this](https://developers.glean.com/docs/indexing_api_datasource_category/) for more details."""

    UNCATEGORIZED = "UNCATEGORIZED"
    TICKETS = "TICKETS"
    CRM = "CRM"
    PUBLISHED_CONTENT = "PUBLISHED_CONTENT"
    COLLABORATIVE_CONTENT = "COLLABORATIVE_CONTENT"
    QUESTION_ANSWER = "QUESTION_ANSWER"
    MESSAGING = "MESSAGING"
    CODE_REPOSITORY = "CODE_REPOSITORY"
    CHANGE_MANAGEMENT = "CHANGE_MANAGEMENT"
    PEOPLE = "PEOPLE"
    EMAIL = "EMAIL"
    SSO = "SSO"
    ATS = "ATS"
    KNOWLEDGE_HUB = "KNOWLEDGE_HUB"
    EXTERNAL_SHORTCUT = "EXTERNAL_SHORTCUT"
    ENTITY = "ENTITY"
    CALENDAR = "CALENDAR"


class HideBuiltInFacet(str, Enum):
    TYPE = "TYPE"
    TAG = "TAG"
    AUTHOR = "AUTHOR"
    OWNER = "OWNER"


class CustomDatasourceConfigConnectorType(str, Enum):
    r"""The source from which document content was pulled, e.g. an API crawl or browser history"""

    API_CRAWL = "API_CRAWL"
    BROWSER_CRAWL = "BROWSER_CRAWL"
    BROWSER_HISTORY = "BROWSER_HISTORY"
    BUILTIN = "BUILTIN"
    FEDERATED_SEARCH = "FEDERATED_SEARCH"
    PUSH_API = "PUSH_API"
    WEB_CRAWL = "WEB_CRAWL"
    NATIVE_HISTORY = "NATIVE_HISTORY"


class CustomDatasourceConfigTypedDict(TypedDict):
    r"""Structure describing config properties of a custom datasource"""

    name: str
    r"""Unique identifier of datasource instance to which this config applies."""
    display_name: NotRequired[str]
    r"""The user-friendly instance label to display. If omitted, falls back to the title-cased `name`."""
    datasource_category: NotRequired[DatasourceCategory]
    r"""The type of this datasource. It is an important signal for relevance and must be specified and cannot be UNCATEGORIZED. Please refer to [this](https://developers.glean.com/docs/indexing_api_datasource_category/) for more details."""
    url_regex: NotRequired[str]
    r"""Regular expression that matches URLs of documents of the datasource instance. The behavior for multiple matches is non-deterministic. **Note: `urlRegex` is a required field for non-entity datasources, but not required if the datasource is used to push custom entities (ie. datasources where isEntityDatasource is false). Please add a regex as specific as possible to this datasource instance.**"""
    icon_url: NotRequired[str]
    r"""The URL to an image to be displayed as an icon for this datasource instance. Must have a transparency mask. SVG are recommended over PNG. Public, scio-authenticated and Base64 encoded data URLs are all valid (but not third-party-authenticated URLs)."""
    object_definitions: NotRequired[List[ObjectDefinitionTypedDict]]
    r"""The list of top-level `objectType`s for the datasource."""
    suggestion_text: NotRequired[str]
    r"""Example text for what to search for in this datasource"""
    home_url: NotRequired[str]
    r"""The URL of the landing page for this datasource instance. Should point to the most useful page for users, not the company marketing page."""
    crawler_seed_urls: NotRequired[List[str]]
    r"""This only applies to WEB_CRAWL and BROWSER_CRAWL datasources. Defines the seed URLs for crawling."""
    icon_dark_url: NotRequired[str]
    r"""The URL to an image to be displayed as an icon for this datasource instance in dark mode. Must have a transparency mask. SVG are recommended over PNG. Public, scio-authenticated and Base64 encoded data URLs are all valid (but not third-party-authenticated URLs)."""
    hide_built_in_facets: NotRequired[List[HideBuiltInFacet]]
    r"""List of built-in facet types that should be hidden for the datasource."""
    canonicalizing_url_regex: NotRequired[List[CanonicalizingRegexTypeTypedDict]]
    r"""A list of regular expressions to apply to an arbitrary URL to transform it into a canonical URL for this datasource instance. Regexes are to be applied in the order specified in this list."""
    canonicalizing_title_regex: NotRequired[List[CanonicalizingRegexTypeTypedDict]]
    r"""A list of regular expressions to apply to an arbitrary title to transform it into a title that will be displayed in the search results"""
    redlist_title_regex: NotRequired[str]
    r"""A regex that identifies titles that should not be indexed"""
    connector_type: NotRequired[CustomDatasourceConfigConnectorType]
    quicklinks: NotRequired[List[QuicklinkTypedDict]]
    r"""List of actions for this datasource instance that will show up in autocomplete and app card, e.g. \"Create new issue\" for jira"""
    render_config_preset: NotRequired[str]
    r"""The name of a render config to use for displaying results from this datasource. Any well known datasource name may be used to render the same as that source, e.g. `web` or `gdrive`. Please refer to [this](https://developers.glean.com/docs/rendering_search_results/) for more details"""
    aliases: NotRequired[List[str]]
    r"""Aliases that can be used as `app` operator-values."""
    is_on_prem: NotRequired[bool]
    r"""Whether or not this datasource is hosted on-premise."""
    trust_url_regex_for_view_activity: NotRequired[bool]
    r"""True if browser activity is able to report the correct URL for VIEW events. Set this to true if the URLs reported by Chrome are constant throughout each page load. Set this to false if the page has Javascript that modifies the URL during or after the load."""
    include_utm_source: NotRequired[bool]
    r"""If true, a utm_source query param will be added to outbound links to this datasource within Glean."""
    strip_fragment_in_canonical_url: NotRequired[bool]
    r"""If true, the fragment part of the URL will be stripped when converting to a canonical url."""
    identity_datasource_name: NotRequired[str]
    r"""If the datasource uses another datasource for identity info, then the name of the datasource. The identity datasource must exist already."""
    product_access_group: NotRequired[str]
    r"""If the datasource uses a specific product access group, then the name of that group."""
    is_user_referenced_by_email: NotRequired[bool]
    r"""whether email is used to reference users in document ACLs and in group memberships."""
    is_entity_datasource: NotRequired[bool]
    r"""True if this datasource is used to push custom entities."""
    is_test_datasource: NotRequired[bool]
    r"""True if this datasource will be used for testing purpose only. Documents from such a datasource wouldn't have any effect on search rankings."""


class CustomDatasourceConfig(BaseModel):
    r"""Structure describing config properties of a custom datasource"""

    name: str
    r"""Unique identifier of datasource instance to which this config applies."""

    display_name: Annotated[Optional[str], pydantic.Field(alias="displayName")] = None
    r"""The user-friendly instance label to display. If omitted, falls back to the title-cased `name`."""

    datasource_category: Annotated[
        Optional[DatasourceCategory], pydantic.Field(alias="datasourceCategory")
    ] = DatasourceCategory.UNCATEGORIZED
    r"""The type of this datasource. It is an important signal for relevance and must be specified and cannot be UNCATEGORIZED. Please refer to [this](https://developers.glean.com/docs/indexing_api_datasource_category/) for more details."""

    url_regex: Annotated[Optional[str], pydantic.Field(alias="urlRegex")] = None
    r"""Regular expression that matches URLs of documents of the datasource instance. The behavior for multiple matches is non-deterministic. **Note: `urlRegex` is a required field for non-entity datasources, but not required if the datasource is used to push custom entities (ie. datasources where isEntityDatasource is false). Please add a regex as specific as possible to this datasource instance.**"""

    icon_url: Annotated[Optional[str], pydantic.Field(alias="iconUrl")] = None
    r"""The URL to an image to be displayed as an icon for this datasource instance. Must have a transparency mask. SVG are recommended over PNG. Public, scio-authenticated and Base64 encoded data URLs are all valid (but not third-party-authenticated URLs)."""

    object_definitions: Annotated[
        Optional[List[ObjectDefinition]], pydantic.Field(alias="objectDefinitions")
    ] = None
    r"""The list of top-level `objectType`s for the datasource."""

    suggestion_text: Annotated[
        Optional[str], pydantic.Field(alias="suggestionText")
    ] = None
    r"""Example text for what to search for in this datasource"""

    home_url: Annotated[Optional[str], pydantic.Field(alias="homeUrl")] = None
    r"""The URL of the landing page for this datasource instance. Should point to the most useful page for users, not the company marketing page."""

    crawler_seed_urls: Annotated[
        Optional[List[str]], pydantic.Field(alias="crawlerSeedUrls")
    ] = None
    r"""This only applies to WEB_CRAWL and BROWSER_CRAWL datasources. Defines the seed URLs for crawling."""

    icon_dark_url: Annotated[Optional[str], pydantic.Field(alias="iconDarkUrl")] = None
    r"""The URL to an image to be displayed as an icon for this datasource instance in dark mode. Must have a transparency mask. SVG are recommended over PNG. Public, scio-authenticated and Base64 encoded data URLs are all valid (but not third-party-authenticated URLs)."""

    hide_built_in_facets: Annotated[
        Optional[List[HideBuiltInFacet]], pydantic.Field(alias="hideBuiltInFacets")
    ] = None
    r"""List of built-in facet types that should be hidden for the datasource."""

    canonicalizing_url_regex: Annotated[
        Optional[List[CanonicalizingRegexType]],
        pydantic.Field(alias="canonicalizingURLRegex"),
    ] = None
    r"""A list of regular expressions to apply to an arbitrary URL to transform it into a canonical URL for this datasource instance. Regexes are to be applied in the order specified in this list."""

    canonicalizing_title_regex: Annotated[
        Optional[List[CanonicalizingRegexType]],
        pydantic.Field(alias="canonicalizingTitleRegex"),
    ] = None
    r"""A list of regular expressions to apply to an arbitrary title to transform it into a title that will be displayed in the search results"""

    redlist_title_regex: Annotated[
        Optional[str], pydantic.Field(alias="redlistTitleRegex")
    ] = None
    r"""A regex that identifies titles that should not be indexed"""

    connector_type: Annotated[
        Optional[CustomDatasourceConfigConnectorType],
        pydantic.Field(alias="connectorType"),
    ] = None

    quicklinks: Optional[List[Quicklink]] = None
    r"""List of actions for this datasource instance that will show up in autocomplete and app card, e.g. \"Create new issue\" for jira"""

    render_config_preset: Annotated[
        Optional[str], pydantic.Field(alias="renderConfigPreset")
    ] = None
    r"""The name of a render config to use for displaying results from this datasource. Any well known datasource name may be used to render the same as that source, e.g. `web` or `gdrive`. Please refer to [this](https://developers.glean.com/docs/rendering_search_results/) for more details"""

    aliases: Optional[List[str]] = None
    r"""Aliases that can be used as `app` operator-values."""

    is_on_prem: Annotated[Optional[bool], pydantic.Field(alias="isOnPrem")] = None
    r"""Whether or not this datasource is hosted on-premise."""

    trust_url_regex_for_view_activity: Annotated[
        Optional[bool], pydantic.Field(alias="trustUrlRegexForViewActivity")
    ] = True
    r"""True if browser activity is able to report the correct URL for VIEW events. Set this to true if the URLs reported by Chrome are constant throughout each page load. Set this to false if the page has Javascript that modifies the URL during or after the load."""

    include_utm_source: Annotated[
        Optional[bool], pydantic.Field(alias="includeUtmSource")
    ] = None
    r"""If true, a utm_source query param will be added to outbound links to this datasource within Glean."""

    strip_fragment_in_canonical_url: Annotated[
        Optional[bool], pydantic.Field(alias="stripFragmentInCanonicalUrl")
    ] = True
    r"""If true, the fragment part of the URL will be stripped when converting to a canonical url."""

    identity_datasource_name: Annotated[
        Optional[str], pydantic.Field(alias="identityDatasourceName")
    ] = None
    r"""If the datasource uses another datasource for identity info, then the name of the datasource. The identity datasource must exist already."""

    product_access_group: Annotated[
        Optional[str], pydantic.Field(alias="productAccessGroup")
    ] = None
    r"""If the datasource uses a specific product access group, then the name of that group."""

    is_user_referenced_by_email: Annotated[
        Optional[bool], pydantic.Field(alias="isUserReferencedByEmail")
    ] = None
    r"""whether email is used to reference users in document ACLs and in group memberships."""

    is_entity_datasource: Annotated[
        Optional[bool], pydantic.Field(alias="isEntityDatasource")
    ] = False
    r"""True if this datasource is used to push custom entities."""

    is_test_datasource: Annotated[
        Optional[bool], pydantic.Field(alias="isTestDatasource")
    ] = False
    r"""True if this datasource will be used for testing purpose only. Documents from such a datasource wouldn't have any effect on search rankings."""
